#include <stdlib.h>
#include <math.h>
#include "emapf.h"

/*
 * cg2cll.c  - source file for conformal mapping function utility.
 * Written 12/21/94 by
 * Dr. Albion Taylor
 * NOAA / OAR / ARL                  Phone: (301) 713-0295 ext 132
 * Rm. 3151, 1315 East-West Highway  Fax:   (301) 713-0119
 * Silver Spring, MD 20910           E-mail: ADTaylor@arlrisc.ssmc.noaa.gov
 */

void C_cg2cll(maparam * stcprm,double lat, double longit,
	    double ug,double vg, double * ue,double * vn) {
double along = cPeriodic(longit - stcprm->reflon, -180., 180.);
double rot,slong,clong,xpolg,ypolg;
  rot = RADPDEG * (-stcprm->gamma * along);
  slong = sin(rot); clong = cos(rot);
  xpolg = slong * stcprm->crotate + clong * stcprm->srotate;
  ypolg = clong * stcprm->crotate - slong * stcprm->srotate;
  *ue = ypolg * ug - xpolg * vg;
  *vn = ypolg * vg + xpolg * ug;
}

void C_cg2wll(maparam * stcprm,double lat, double longit,
	    double ug,double vg, double * ue,double * vn) {
double along = cPeriodic(longit - stcprm->reflon, -180., 180.);
double rot,slong,clong,xpolg,ypolg;
  if ( lat > 89. ) {
    rot = RADPDEG * ( -stcprm->gamma * along + longit - 180.);
  } else {
    if (lat < -89.) {
      rot = RADPDEG * (-stcprm->gamma * along - longit);
    } else {
      rot = RADPDEG * (-stcprm->gamma * along);
    }
  }
  slong = sin(rot); clong = cos(rot);
  xpolg = slong * stcprm->crotate + clong * stcprm->srotate;
  ypolg = clong * stcprm->crotate - slong * stcprm->srotate;
  *ue = ypolg * ug - xpolg * vg;
  *vn = ypolg * vg + xpolg * ug;
}

#ifdef DO_FORTCALLS
INT4v CG2CLL(maparam * stcprm,REAL4 lat,REAL4 longit,
            REAL4 ug,REAL4 vg, REAL4 ue,REAL4 vn) {
double ues,vns;
  C_cg2cll(stcprm,* lat,* longit, * ug,* vg, &ues, &vns);
  *ue = ues; *vn = vns;
  return 0;
}

INT4v CG2WLL(maparam * stcprm,REAL4 lat,REAL4 longit,
            REAL4 ug,REAL4 vg, REAL4 ue,REAL4 vn) {
double ues,vns;
  C_cg2wll(stcprm,* lat,* longit, * ug,* vg, &ues, &vns) ;
  *ue = ues, *vn = vns;
  return 0;
}

#endif /*DO_FORTCALLS*/
